@@ -329,7 +329,7 @@ static int Rpar2jvalue(JNIEnv *env, SEXP par, jvalue *jpar, sig_buffer_t *sig, i
       }
     } else if (TYPEOF(e)==VECSXP || TYPEOF(e)==S4SXP) {
       _dbg(rjprintf(" generic vector of length %d\n", LENGTH(e)));
-      if (inherits(e,"jobjRef")||inherits(e,"jarrayRef")) {
+      if (IS_JOBJREF(e)) {
 	jobject o=(jobject)0;
 	const char *jc=0;
 	SEXP n=getAttrib(e, R_NamesSymbol);
@@ -349,7 +349,7 @@ static int Rpar2jvalue(JNIEnv *env, SEXP par, jvalue *jpar, sig_buffer_t *sig, i
 	  sclass=GET_SLOT(e, install("jclass"));
 	  if (sclass && TYPEOF(sclass)==STRSXP && LENGTH(sclass)==1)
 	    jc=CHAR_UTF8(STRING_ELT(sclass,0));
-	  if (inherits(e, "jarrayRef") && jc && !*jc) {
+	  if (IS_JARRAYREF(e) && jc && !*jc) {
 	    /* if it's jarrayRef with jclass "" then it's an uncast array - use sig instead */
 	    sclass=GET_SLOT(e, install("jsig"));
 	    if (sclass && TYPEOF(sclass)==STRSXP && LENGTH(sclass)==1)
@@ -767,12 +767,50 @@ HIDE SEXP new_jobjRef(JNIEnv *env, jobject o, const char *klass) {
   return oo;
 }
 
+/** 
+ * creates a new jclassName object. similar to what the jclassName
+ * function does in the R side
+ *
+ * @param env pointer to the jni env
+ * @param cl Class instance
+ */
+HIDE SEXP new_jclassName(JNIEnv *env, jobject/*Class*/ cl ) {
+  SEXP oo = NEW_OBJECT(MAKE_CLASS("jclassName"));
+  if (!inherits(oo, "jclassName"))
+    error("unable to create jclassName object");
+  PROTECT(oo);
+  SET_SLOT(oo, install("name"), getName(env, cl) );
+  SET_SLOT(oo, install("jobj"), new_jobjRef( env, cl, "java/lang/Class" ) );
+  UNPROTECT(1);
+  return oo;
+}
+
+/** Calls the Class.getName method and return the result as an R STRSXP */
+HIDE SEXP getName( JNIEnv *env, jobject/*Class*/ cl){
+	char cn[128];
+	
+	jstring r = (*env)->CallObjectMethod(env, cl, mid_getName);
+  
+	cn[127]=0; *cn=0;
+  	int sl = (*env)->GetStringLength(env, r);
+  	if (sl>127) {
+  	  error("class name is too long");
+  	}
+  	if (sl) (*env)->GetStringUTFRegion(env, r, 0, sl, cn);
+  	char *c=cn; while(*c) { if (*c=='.') *c='/'; c++; }
+
+	SEXP res = PROTECT( mkString(cn ) ); 
+	releaseObject(env, r);
+	UNPROTECT(1); /* res */
+	return res;
+}
+
 static SEXP new_jarrayRef(JNIEnv *env, jobject a, const char *sig) {
   /* it is too tedious to try to do this in C, so we use 'new' R function instead */
   /* SEXP oo = eval(LCONS(install("new"),LCONS(mkString("jarrayRef"),R_NilValue)), R_GlobalEnv); */
   SEXP oo = NEW_OBJECT(MAKE_CLASS("jarrayRef"));
   /* .. and set the slots in C .. */
-  if (!inherits(oo, "jarrayRef"))
+  if (! IS_JARRAYREF(oo) )
     error("unable to create an array");
   PROTECT(oo);
   SET_SLOT(oo, install("jobj"), j2SEXP(env, a, 1));
@@ -782,22 +820,68 @@ static SEXP new_jarrayRef(JNIEnv *env, jobject a, const char *sig) {
   return oo;
 }
 
+/**
+ * Creates a reference to a rectangular java array.
+ *
+ * @param env 
+ * @param a the java object
+ * @param sig signature (class of the array object)
+ * @param dim dimension vector
+ */
+static SEXP new_jrectRef(JNIEnv *env, jobject a, const char *sig, SEXP dim ) {
+  /* it is too tedious to try to do this in C, so we use 'new' R function instead */
+  /* SEXP oo = eval(LCONS(install("new"),LCONS(mkString("jrectRef"),R_NilValue)), R_GlobalEnv); */
+  SEXP oo = NEW_OBJECT(MAKE_CLASS("jrectRef"));
+  /* .. and set the slots in C .. */
+  if (! IS_JRECTREF(oo) )
+    error("unable to create an array");
+  PROTECT(oo);
+  SET_SLOT(oo, install("jobj"), j2SEXP(env, a, 1));
+  SET_SLOT(oo, install("jclass"), mkString(sig));
+  SET_SLOT(oo, install("jsig"), mkString(sig));
+  SET_SLOT(oo, install("dimension"), dim);
+  
+  UNPROTECT(1); // oo
+  return oo;
+}
+
+/**
+ * Creates a rectangular array reference of dimension 1
+ *
+ * @param length the number of objects in the array
+ */
+static SEXP new_jrectRef1(JNIEnv *env, jobject a, const char *sig, int length ) {
+	SEXP dim = allocVector( INTSXP, 1) ;
+	PROTECT(dim); 
+	INTEGER(dim)[0] = length ;
+	UNPROTECT(1); // dim 
+	return new_jrectRef( env, a, sig, dim ) ;
+}
+
+// this does not take care of multi dimensional arrays properly
+
+/**
+ * Creates a one dimensionnal java array
+ *
+ * @param an R list or vector
+ * @param cl the class name
+ */
 REPC SEXP RcreateArray(SEXP ar, SEXP cl) {
   JNIEnv *env=getJNIEnv();
   
   if (ar==R_NilValue) return R_NilValue;
   switch(TYPEOF(ar)) {
-  case INTSXP:
+  case INTSXP:                                
     {
       if (inherits(ar, "jbyte")) {
 	jbyteArray a = newByteArrayI(env, INTEGER(ar), LENGTH(ar));
 	if (!a) error("unable to create a byte array");
-	return new_jarrayRef(env, a, "[B");
+	return new_jarrayRef(env, a, "[B" ) ;
       } else if (inherits(ar, "jchar")) {
 	jcharArray a = newCharArrayI(env, INTEGER(ar), LENGTH(ar));
 	if (!a) error("unable to create a char array");
-	return new_jarrayRef(env, a, "[C");
-      } else {
+	return new_jarrayRef(env, a, "[C" ); 
+	  } else {
 	jintArray a = newIntArray(env, INTEGER(ar), LENGTH(ar));
 	if (!a) error("unable to create an integer array");
 	return new_jarrayRef(env, a, "[I");
@@ -850,7 +934,8 @@ REPC SEXP RcreateArray(SEXP ar, SEXP cl) {
 	SEXP e = VECTOR_ELT(ar, i);
 	if (e != R_NilValue &&
 	    !inherits(e, "jobjRef") &&
-	    !inherits(e, "jarrayRef"))
+	    !inherits(e, "jarrayRef") && 
+	    !inherits(e, "jrectRef") )
 	  error("Cannot create a Java array from a list that contains anything other than Java object references.");
 	i++;
       }
